(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, ENAC/CENA          *)
(*                                                                     *)
(***********************************************************************)
(* $Id: fcl_ac6.ml,v 1.9 2001/01/18 09:52:33 brisset Exp $ *)

open Fcl_var

class ac6 (v1 : Fd.t) (v2 : Fd.t) nogoods =
  let minv1 = Fd.min v1 in
  let intervalv1 = Fd.max v1 - Fd.min v1 + 1 in
  let val_of_index i = i + minv1
  and index_of_val v = v - minv1 in
  let nogoodsv1 =
    Array.init
      intervalv1
      ~f:(fun i ->
	List.fold_right
	  ~f:(fun (i1,j) acc -> if i1 = val_of_index i then j::acc else acc)
	  nogoods
	  ~init:[]) in
  let nextsupport j i vi = (* cherche dans vi, un i'>i, support de j *)
    let rec loop i1 =
      if i1 > Fd.max vi
      then raise Not_found
      else if Fd.member vi i1 && not (List.mem j nogoodsv1.(index_of_val(i1)))
      then i1
      else loop (i1+1) in
    loop (i+1) in
  let call_nextsupport j vi s =
    if Fd.member v2 j then
      try
      	let i = index_of_val (nextsupport j vi v1) in
	Fcl_stak.array_set s i (j::s.(i))
      with
      	Not_found ->
	  match Fd.value v2 with
	    Unk v2_ -> Fd.refine v2 (Fcl_domain.remove j (Attr.dom v2_))
	  | Val vj -> assert (vj = j); Fcl_stak.fail "ac6" in
  let s = 
    let s = Array.create intervalv1 [] in
    Fd.iter v2 (fun j -> call_nextsupport j (minv1-1) s);
    s in
  let last_domain =
    Fcl_stak.ref (match Fd.value v1 with
      Unk v1_ -> Attr.dom v1_
    | Val vi -> Fcl_domain.create [vi]) in

  object
    inherit Fcl_cstr.t Fcl_cstr.normal
      
    method name = "ac6"
    method delay x =
      delay [Attr.on_refine] v1 x;
      delay [Attr.on_refine] v2 x
	
    method update =
      if not (Fcl_stak.get solved) then begin
	let new_domain = 
      	  match Fd.value v1 with
	    Unk v1_ -> Attr.dom v1_
	  | Val vi -> Fcl_stak.set solved true; Fcl_domain.create [vi] in
	let removed_values =
	  Fcl_domain.difference (Fcl_stak.get last_domain) new_domain in
	Fcl_stak.set last_domain new_domain;
	Fcl_domain.iter
	  (fun i ->
	    let si = s.(index_of_val i) in
	    List.iter ~f:(fun j -> call_nextsupport j i s) si
          (***; s.(index_of_val i) n'est plus utile. Faut-t-il le virer ???
	     Fcl_stak.trail (fun () -> s.(index_of_val i) <- si);
	     s.(index_of_val i) <- []***))
	  removed_values
      end
  end

open Fcl_arith

let cstr var1 (var2 : Fcl_var.Fd.t) ng =
  let c1 = Fcl_reify.boolean ~delay_on_negation:false (new ac6 var1 var2 ng)
  and c2 = Fcl_reify.boolean ~delay_on_negation:false (new ac6 var2 var1 (List.map ~f:(fun (x,y)-> (y,x)) ng)) in

  fd2e c1 +~ fd2e c2 =~ i2e 2
