(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, ENAC/CENA          *)
(*                                                                     *)
(***********************************************************************)
(* $Id: fcl_reify.mli,v 1.15 2001/01/15 13:16:42 barnier Exp $ *)

(* Module [Reify]: constraints reification *)

val boolean : ?delay_on_negation:bool -> Fcl_cstr.t -> Fcl_var.Fd.t
 (* [boolean (?delay_on_negation:true) c] returns a boolean (0..1)
 variable associated to the constraint [c]. The constraint is verified
 iff the boolean is equal to 1. The waking conditions of the contraint
 relating [c] and the boolean are the ones set by the [delay] method
 of [c] (set by the [delay] argument of [Cstr.create]). If the optional
 argument [delay_on_negation] is set to [true], the new constraint is
 also attached to the events of the negation of [c] (i.e. the
 constraint returned by the [not] method of [c]), otherwise it is not.
 [delay_on_negation] default value is [true]. *)

val cstr : ?delay_on_negation:bool -> Fcl_cstr.t -> Fcl_var.Fd.t -> Fcl_cstr.t
 (* [cstr delay_on_negation c b] is equivalent to the
    constraint [boolean ?delay_on_negation c =~ b].
    [delay_on_negation] default value is [true]. *)


val (&&~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val (||~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val (=>~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val (<=>~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val not : Fcl_cstr.t -> Fcl_cstr.t
(* Logical operations on constraints (which are reified), namely
   and, or, implies, equivalent, not. *)
