(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, ENAC/CENA          *)
(*                                                                     *)
(***********************************************************************)
(* $Id: fcl_stak.ml,v 1.7 2001/01/17 21:21:23 brisset Exp $ *)

class virtual gl = object
  method virtual name : string
  method virtual call : gl option
end

type level = int

type 'a ref = 'a Pervasives.ref

exception Empty_stack
exception Level_not_found of int

type s = Alive | Cut
type lev = {
    level: level;
    success: gl list;
    mutable failure: stack;
    mutable status: s
  }
and  stack = Level of lev
           | Empty
           | Trail of (unit -> unit) * stack

let stack = ref Empty
let reset () = stack := Empty
let bottom_level =  Fcl_misc.gen_int ()
let save x =
  let l = Fcl_misc.gen_int () in
  stack := Level {level=l;success=x; failure= !stack; status=Alive};
  l

let backtrack () =
  let rec bt = function
      Level {status=Cut;failure=s} -> bt s (* niveau coup *)
    | Level {success=x;failure=s} -> stack := s; x
    | Empty ->  reset (); raise Empty_stack
    | Trail (undo, s) -> undo (); bt s in
  bt !stack


let backtrack_all () =
  let rec bt = function
      Level {failure=s} -> bt s
    | Empty -> reset ()
    | Trail (undo, s) -> undo (); bt s in
  bt !stack
    

let size () =
  let rec count = function
      Level {failure=s} -> count s
    | Empty ->  0
    | Trail (_undo, s) -> 1 + count s
    in count !stack

let trail undo = if !stack <> Empty then stack := Trail (undo, !stack)


let cut level =
  if level = bottom_level then
    reset ()
  else
    let rec c = function
      	Level ({level=l; failure=f} as ll)->
      	  if l <> level then (ll.status <- Cut; c f)
      | Empty -> raise (Level_not_found level)
      | Trail (_,s) -> c s in
    c !stack

let cut_bottom level =
  if level <> bottom_level then
    let rec c = function
      	Level ({level=l; failure=f} as ll)->
      	  if l <> level then c f
	  else begin
	    ll.failure <- Empty;
	    ll.status <- Cut end
      | Empty -> raise (Level_not_found level)
      | Trail (_,s) -> c s in
    c !stack
      
let level () =
  let rec c = function
      Level {status=s; failure=st; level=l} ->
	if s = Cut then c st else l
    | Empty -> bottom_level
    | Trail (_, s) -> c s in
  c !stack


let set refb value =
  let old = !refb in
    refb := value; trail (fun () -> refb := old)

let array_set t i v =
  let old = t.(i) in
  t.(i) <- v; trail (fun () -> t.(i) <- old)

let get x = !x
let unsafe_set r x = r := x
let ref x = ref x

exception Fail of string
let fail x = raise (Fail x)
